@@ -25,13 +25,25 @@ enum class CandidateLayoutHint { NotSet, Vertical, Horizontal };
 
 class CandidateWordPrivate;
 
+/// Base class of candidate word.
 class FCITXCORE_EXPORT CandidateWord {
 public:
     CandidateWord(Text text = {});
     virtual ~CandidateWord();
+    /**
+     * Called when candidate is selected by user.
+     *
+     * @param inputContext the associated input context for the candidate.
+     */
     virtual void select(InputContext *inputContext) const = 0;
 
     const Text &text() const;
+    /**
+     * Whether the candidate is only a place holder.
+     *
+     * If candidate is a place holder, it will not be displayed by UI, but it
+     * will still take one place in the candidate list.
+     */
     bool isPlaceHolder() const;
     bool hasCustomLabel() const;
     const Text &customLabel() const;
@@ -121,7 +133,7 @@ public:
     }
 
     template <typename CandidateWordType, typename... Args>
-    void append(Args &&... args) {
+    void append(Args &&...args) {
         append(
             std::make_unique<CandidateWordType>(std::forward<Args>(args)...));
     }
@@ -162,6 +174,9 @@ class CommonCandidateListPrivate;
 
 enum class CursorPositionAfterPaging { SameAsLast, DonotChange, ResetToFirst };
 
+/**
+ * A common simple candidate list that serves most of the purpose.
+ */
 class FCITXCORE_EXPORT CommonCandidateList : public CandidateList,
                                              public PageableCandidateList,
                                              public ModifiableCandidateList,
@@ -171,11 +186,39 @@ public:
     ~CommonCandidateList();
 
     void clear();
+
+    /**
+     * Set the label of candidate list.
+     *
+     * The labels less than 10 will be automatically filled with to empty ones
+     * up to 10 to be more error prone.
+     *
+     * @param labels list of labels.
+     *
+     * @since 5.0.4
+     */
+    void setLabels(const std::vector<std::string> &labels = {});
+
+    /**
+     * Set the label of candidate list by key.
+     *
+     * @param keyList list of selection key
+     */
     void setSelectionKey(const KeyList &keyList);
+
     void setPageSize(int size);
     int pageSize() const;
     void setLayoutHint(CandidateLayoutHint hint);
     void setGlobalCursorIndex(int index);
+    /**
+     * Return Global cursor index.
+     *
+     * -1 means it is not selected.
+     *
+     * @return cursor index.
+     * @since 5.0.4
+     */
+    int globalCursorIndex() const;
 
     // CandidateList
     const fcitx::Text &label(int idx) const override;
